# Wrapper for ALICE, adding user commands to modify the score and volume,
# and provide help. 
# This class can function both as a command-line program and 
# as an IRC bot when it is loaded by the RedTurnip IRC bot framework.

begin 
   require 'alice' # for when we start from the command line
rescue LoadError
   require 'mod/bot/alice' # for when we start from RedTurnip 
end

class ALICEBot 

   attr_accessor :MY_NAME

   def initialize
      print "Hello\n"
      @agent = ALICE.new
      @THRESHHOLD = 1
      @SCORE_MOD = 0
      @MAX_RESPONSE = 10000
      @MY_NAME = "(?i)(?:alice|alicebot|aliceagent)"
      @rules = []
      @rules.push("if response =~ /JUDGE/ then response.gsub!(/JUDGE/, 'dude') end")
      @rules.push("if response =~ /YourBot/ then response.gsub!(/YourBot/, 'me') end")
      @rules.push("if response =~ /ALICE/ then score = 0 end")
      @rules.push("if response =~ /( +)/ then response.gsub!(/( +)/, ' ') end")
      @rules.push("if response =~ /AIML/ then score = 0 end")
      @rules.push("if response =~ /default category/ then score = 0 end")
      @SAFE_LEVEL = "low"
      @Low_safe_level_words = ["low", "default", "0", "off"]
      @High_safe_level_words = ["high", "on", "1"]
   end

   def getResponse(input)

      input = input.to_s
      print 'INPUT==' + input + "\n" unless $0 == __FILE__

      if input =~ /^what is your name?/
         name = @MY_NAME
         name.gsub!(/\(\?i\)\(\?\:/, '')
         name.gsub!(/\|/, " or ")
         name.gsub!(/\)/, '')
         return name
      end
      # Set threshhold.
      if input =~ /^#{@MY_NAME}[:,!]* (quiet|be quiet|stfu|shut.*up|soft)/
         @THRESHHOLD = @THRESHHOLD + 1
         @SCORE_MOD -= 1
         return "My threshhold is now " << @THRESHHOLD.to_s << ", and my score_mod is #{@SCORE_MOD}."
      end
      if input =~ /^#{@MY_NAME}[:,!]* (speak.*up|loud|louder|i can't hear you)/
         @THRESHHOLD = @THRESHHOLD - 1
         @SCORE_MOD += 1
         return "My threshold is now " << @THRESHHOLD.to_s << ", and my score_mod is #{@SCORE_MOD}."
      end
      # Return threshhold
      if input =~ /^#{@MY_NAME}[:,!]* what is your (threshhold|threshold|volume)/ or input =~ /^#{@MY_NAME}[:,!]* (?:threshold|threshhold|volume)/i
         return "My threshhold is currently " << @THRESHHOLD.to_s
      end

      # Provide help message.
      if input =~ /^#{@MY_NAME}[:,!]* help/
         return "I am ALICE by Dr. Richard Wallace."
      end

      # trap restart
      case input
        when /^#{@MY_NAME}[:,!]* restart/, /^restart/
          return "That command is disabled." # for safety
      end

      # add new if-then rules to post-process responses
      case input
        when /^#{@MY_NAME}[:,!]* (if .*? =~ .*? then .*)$/
          return add_if_rule($1)
      end

      # show the rules
      case input
        when /^#{@MY_NAME}[:,!]*.* show.* rules/
          return show_rules[0]
      end

      # delete all rules, or one rule by its index
      case input
        when /^#{@MY_NAME}[:,!]*.* delete.* rules/ 
          return delete_rules
        when /^#{@MY_NAME}[:,!]*.* delete.* rule.* (\d+)/
          return delete_rule($1.to_i)
      end

      # set safe level
      case input
        when /^#{@MY_NAME}[:,! ]*(?:set )?.*safe(?:ty)? level (?:to )?(.*)/
          return set_safe_level($1)
      end

      # get safe level
      case input 
        when /^#{@MY_NAME}[:,! ]*what is.*?safe(?:ty)? level/
          return  get_safe_level
      end

      # Get the agent's response.
      if input =~ /^#{@MY_NAME}[:,! ]*(.*)/ then input = $1 end
      r, s = @agent.getResponse(input)
      if s == nil then s = rand(3) + 1 end
      if s > 3 then s -= 1 end
      r, s = apply_if_rules(r,s) 
      s += @SCORE_MOD
      if r.to_s.size > @MAX_RESPONSE
         r = r[0..@MAX_RESPONSE]
      end
      puts "\n[alicebot] score==" << s.to_s << "; threshhold==" << @THRESHHOLD.to_s
      if s > @THRESHHOLD 
        if $0 == __FILE__ then return r end
        puts "RESPONSE: #{r}"
        return [r, s]
      else
        puts "r==#{r}"
      end

      return ""
   end

   # Respond to natural language method calls.
   def method_missing input
     return getResponse(input.to_s)
   end

   # add an if-then rule
   def add_if_rule(rule = '')
     if rule == '' then return "No rule found." end
     if @SAFE_LEVEL =~ /#{@Low_safe_level_words.join('|')}$/ 
       @rules.push(rule)
       return "Okay I have added #{rule}."
     end
     return "I can't add any if rules because my safe level is #{@SAFE_LEVEL}."
   end

   # apply if-then rules
   def apply_if_rules(response, score)
     @rules.each { |rule|
       puts "rule==#{rule}; response==#{response}; score==#{score}"
       begin
         eval(rule)
       rescue; puts "Can't apply rule: #{rule}"; next; end
     }
     #puts "returning: response==#{response}; score==#{score}"
     return [response, score]
   end

   # return the rules
   def show_rules
     return [@rules.join("\n"), 4]
   end

   # delete a rule by its index
   def delete_rule(i)
     if i < 1 or i > @rules.size
       return "The rules go from 1 to #{@rules.size}."
     end
     rule = @rules[i-1]
     @rules.delete_at(i-1)
     return "Okay, \"#{rule}\" deleted."
   end

   # delete all the rules
   def delete_rules
     count = @rules.size
     @rules = []
     return "Okay, I have deleted #{count} rules."
   end

   def set_safe_level(level)
     case level 
       when /\b#{@Low_safe_level_words.join('|')}\b/ 
         @SAFE_LEVEL = "off"
         return "Okay I have turned off the safe level."
       when /\b#{@High_safe_level_words.join('|')}\b/
         @SAFE_LEVEL = "on"
         return "Okay I have turned the safe level on."
     end
     return "I don't understand #{level}."
   end

   def get_safe_level
     return "My safe level is set to #{@SAFE_LEVEL}."
   end # method_def

end


if $0 == __FILE__
  b = ALICEBot.new
  print("\n> "); $stdout.flush
  quitwords = [":q", "quit", "exit", "bye"]
  while (line = gets) !~ /^#{quitwords.join('|')}$/ and line !~ /^$/
    puts "\n#{b.send(line)}"
    print("\n> "); $stdout.flush
  end
  print "Bye!"
end
